"""Module responsible for detecting dates in the text."""

from typing import Any, Dict, List, Tuple

from src.detections import DateDetection
from src.detectors.interface import Detector

from .en import detect_dates_en
from .pl import detect_dates_pl
from .ru import detect_dates_ru


class DateDetector(Detector):
    """Detects dates in the text."""

    def __init__(self, language: str = "pl") -> None:
        """Initializes the detector.

        Args:
            language (str, optional): Language of analyzed texts. Defaults to "pl".

        """
        self._language = language

    def detect(
        self, text: str, annotations: Dict[str, List[Tuple[int, int, Any]]]
    ) -> List[Tuple[int, int, DateDetection]]:
        """Finds dates in the text.

        Args:
            text (str): The text to be searched
            annotations (Dict[str, List[Tuple[int, int, Any]]]): Annotations of the text

        Returns:
            List[Tuple[int, int, DateDetection]]: a list of tuples containing
                (start, end, annotation)

        """
        return find_dates(text, self._language)


def find_dates(text: str, language: str = "pl") -> List[Tuple[int, int, DateDetection]]:
    """Finds dates in the text.

    Args:
        text (str): the text to be searched
        language (str, optional): The language of the text. Defaults to "pl".

    Returns:
        List[Tuple[int, int, DateDetection]]: a list of tuples containing
            (start, end, annotation)

    """
    language_processors = {
        "en": detect_dates_en,
        "pl": detect_dates_pl,
        "ru": detect_dates_ru,
    }

    return language_processors.get(language, detect_dates_en)(text)
