"""Module implementing NERFileNKJPMorphosyntacticDictionary class.

Module responsible for Morphosyntactic dict that uses a tsv file with NER tags
and NKJP-formatted morpho tags.

NKJP-formatted morpho tags differ from the original morpho tags that they don't have
part of speech information. For example:
Nomral morpheus tag: subst:sg:gen:m2
NKJP morpho tag: sg:gen:m2
"""

import random
from typing import List, Optional, Type

from src.detections import DETECTION_CLASSES_MAP, Detection, MorphosyntacticInfoMixin
from src.dictionaries.morphosyntactic.ner_file import NERFileMorphosyntacticDictionary


class NERFileNKJPMorphosyntacticDictionary(NERFileMorphosyntacticDictionary):
    """Morphosyntactic dictionary that uses a tsv file with NER tags as a source.

    NKJP-formatted morpho tags differ from the original morpho tags that they don't have
    part of speech information. For example:
    Nomral morpheus tag: subst:sg:gen:m2
    NKJP morpho tag: sg:gen:m2

    Example of a tsv file:
    name	Aaronom	Aaron	pl:dat:m1
    name	Aaronami	Aaron	pl:inst:m1
    name	Aaronach	Aaron	pl:loc:m1
    country	Apolonie	Apolonia	pl:voc:f
    country	Apolonii	Apolonia	sg:dat:f
    country	Apolonii	Apolonia	pl:gen:f
    country	Apolonii	Apolonia	sg:loc:f
    city	Araba	Arab	sg:gen:m2
    city	Arabie	Arab	sg:voc:m2
    city	Arabem	Arab	sg:inst:m2
    """

    def __init__(
        self,
        dictionary_path: Optional[str] = None,
        always_replace=True,
        remove_first_morpho_subtag=True,
    ) -> None:
        """Initializes NERFileNKJPMorphosyntacticDictionary.

        Args:
            dictionary_path (Optional[str], optional): Path to dictionary tsv file.
                    Defaults to None.
            always_replace (bool, optional): Wheter to replace detection even if no
                    word with matching morpho tag is found. Defaults to True.
            remove_first_morpho_subtag (bool, optional): Wheter to remove first morpho.
                    Defaults to True.

        """
        super().__init__(dictionary_path, always_replace)
        self._remove_first_morpho_subtag = remove_first_morpho_subtag

    def get_supported_detection_classes(self) -> List[Type[Detection]]:
        """Returns a list of supported detection classess.

        Returns:
            List[Type[Detection]]: List of detection classes that are supported

        """
        return [DETECTION_CLASSES_MAP[name] for name in self._dictionary.keys()]

    def get_random_replacement(self, original_entry: Detection) -> Optional[str]:
        """Returns a random replacement of original entry.

        Args:
            original_entry (Detection): Detection that should be replaced. Class
                should have MorphosyntacticInfoMixin


        Returns:
            Optional[str]: Text that should replace the original entry

        """
        original_entry_type = type(original_entry)
        original_entry_type_name = original_entry_type.TYPE_NAME

        result = None

        if issubclass(original_entry_type, MorphosyntacticInfoMixin):
            # THAT IS A HACK FOR NOW FOR CORRUPTED NKJP TAGS IN DICTIONARY
            if self._remove_first_morpho_subtag:
                morpho_tag = ":".join(original_entry.morpho_tag.split(":")[1:])
            else:
                morpho_tag = original_entry.morpho_tag

            if (
                original_entry_type_name in self._dictionary
                and morpho_tag in self._dictionary[original_entry_type_name]
            ):
                result = random.choice(
                    list(
                        self._dictionary[original_entry_type_name][morpho_tag].values()
                    )
                )

        if result is None and self._always_replace:
            random_type = random.choice(list(self._dictionary.keys()))
            random_tag = random.choice(list(self._dictionary[random_type].keys()))

            result = random.choice(
                list(self._dictionary[random_type][random_tag].values())
            )

        return result
