"""Module for replacing NER entities with anonimized version."""

from typing import List, Tuple

from src.detections import Detection
from src.dictionaries.morphosyntactic import MorphosyntacticDictionary
from src.replacers.interface import ReplacerInterface
from src.string_replacements import replace_and_update


class NERReplacer(ReplacerInterface):
    """Replaces any detected NER entities with their morphosyntactic equivalents."""

    def __init__(self, dictionary: MorphosyntacticDictionary):
        """Initializes NERReplacer.

        Args:
            dictionary (MorphosyntacticDictionary): Dictionary used for
                replacing words with their morphosyntactic equivalents.

        """
        self._dictionary = dictionary

    def replace(
        self, text: str, detections: List[Tuple[int, int, Detection]]
    ) -> Tuple[str, List[Tuple[int, int, Detection]]]:
        """Replace detected NER entities in text with anonimized version.

        Eg.: "John Smith is a nice guy" -> "Mark Johnson is a nice guy"

        Args:
            text (str): Text to be processed.
            detections (List[Tuple[int, int, str]]): List of detections.

        Returns:
            Tuple[str, List[Tuple[int, int, str]]]: Text with supported entities
            replaced with anonimized version and list of detections that were
            not processed by this replacer.

        """
        replacements = []
        not_processed = []

        already_replaced = dict()

        for item in detections:
            if type(item[2]) not in self._dictionary.get_supported_detection_classes():
                not_processed.append(item)
                continue

            start, end, detection = item

            key = (text[start:end], type(detection))

            if key not in already_replaced:
                replacement = self._dictionary.get_random_replacement(detection)
                already_replaced[key] = replacement

            if already_replaced[key] is None:
                not_processed.append(item)
            else:
                replacements.append((start, end, already_replaced[key]))

        return replace_and_update(text, replacements, not_processed)
